package com.brilliance.swift.mx2mt;

import com.brilliance.swift.SwiftTransfer;
import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.mt103.Mx2Mt103Creator;
import com.brilliance.swift.mx2mt.mt103Retn.Mx2Mt103RetnCreator;
import com.brilliance.swift.mx2mt.mt192292.Mx2Mtn92Creator;
import com.brilliance.swift.mx2mt.mt196296.Mx2Mtn96Creator;
import com.brilliance.swift.mx2mt.mt202Retn.Mx2Mt202RetnCreator;
import com.brilliance.swift.mx2mt.mt202Retn.Mx2Mt205RetnCreator;
import com.brilliance.swift.mx2mt.mt202cov202.Mx2Mt202Creator;
import com.brilliance.swift.mx2mt.mt210.Mx2Mt210Creator;
import com.brilliance.swift.mx2mt.mt900910.Mx2Mt900910Creator;
import com.brilliance.swift.mx2mt.mt920.Mx2Mt920Creator;
import com.brilliance.swift.mx2mt.mt940950.Mx2Mt940950Creator;
import com.brilliance.swift.mx2mt.mt941942.Mx2Mt942Creator;
import com.brilliance.swift.mx2mt.mtn99.Mx2MtN99Creator;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.XmlUtil;
import com.brilliance.swift.vo.SwiftTranslationReport;
import com.prowidesoftware.swift.model.mx.AbstractMX;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.util.Map;

public class Mx2MtCreatorManager {

    public SwiftTranslationReport mx2Mt(String xml, String fileOutputPath, Map<String, Object> extraMap) throws SwiftException {
        try {
            xml = handlingXml(xml);
            AbstractMX abstractMX = AbstractMX.parse(xml);
            String messageType = (abstractMX.getMxId().getBusinessProcess().name()
                    + "."
                    + abstractMX.getMxId().getFunctionality()
                    + "."
                    + abstractMX.getMxId().getVariant())
                    //+ abstractMX.getMxId().getVersion())
                    .trim();
            Mx2MtContext context = new Mx2MtContext();
            Document document = DocumentHelper.parseText(xml);
            context.set(Mx2MtContextIdentifier.MX_TO_MT_LISTENER_CLASS, new Mx2MtListener()); //注册监听器
            context.set(Mx2MtContextIdentifier.MX_XMl_DOCUMENT, document);
            context.set(Mx2MtContextIdentifier.MX_TO_MT_OUTPUT_FILE_PATH, fileOutputPath);
            context.set(Mx2MtContextIdentifier.MX_OBJ, abstractMX);
            context.set(Mx2MtContextIdentifier.MX_XMl, xml);
            String applicationMode = "O";//代表是收报，默认值
            if (extraMap != null) {
                applicationMode = (String) extraMap.get(Mx2MtConstants.APPLICATION_MODE);
                if (StringUtil.isEmpty(applicationMode)) {
                    String bnkBic = (String)extraMap.get("bnkBic");//得到本行的bicCode
                    if (StringUtil.isNotEmpty(bnkBic) && abstractMX.getAppHdr() != null) {
                        if (bnkBic.equals(abstractMX.getAppHdr().from())) {
                            applicationMode = "I";//如果发报行和本行bic一样，表示是发报
                        }
                    }
                }
            }
            context.putAll(extraMap);
            context.set(Mx2MtConstants.APPLICATION_MODE, StringUtil.isEmpty(applicationMode)?"O":applicationMode);
            AbstractMx2MtCreator creator = getCreator(handlingMessageType(document, messageType));
            creator.setContext(context);
            creator.preProcess();
            creator.withBlock1();
            creator.withBlock2();
            creator.withBlock3();
            creator.withBlock4();
            creator.withBlock5();
            creator.postProcess();
            SwiftTranslationReport str = context.get(SwiftTranslationReport.class);
            return str;
        } catch (DocumentException e) {
            throw new SwiftException(e.getMessage());
        }
    }

    public AbstractMx2MtCreator getCreator(String messageType) {
        if ("pacs.008.001".equals(messageType)) {
            return new Mx2Mt103Creator();
        } else if ("pacs.009.001".equals(messageType)) {
            return new Mx2Mt202Creator();
        }else if("camt.054.001".equals(messageType)){
            return new Mx2Mt900910Creator();
        } else if("camt.053.001".equals(messageType)){
            return new Mx2Mt940950Creator();
        } else if("camt.056.001".equals(messageType)){
            return new Mx2Mtn92Creator();
        } else if("camt.029.001".equals(messageType)){
            return new Mx2Mtn96Creator();
        } else if("camt.052.001".equals(messageType)){
            return new Mx2Mt942Creator();
        } else if("pacs.004.001.MT103".equals(messageType)){
            return new Mx2Mt103RetnCreator();
        } else if("pacs.004.001.MT202".equals(messageType)){
            return new Mx2Mt202RetnCreator();
        } else if("pacs.004.001.MT205".equals(messageType)){
            return new Mx2Mt205RetnCreator();
        } else if("camt.057.001".equals(messageType)){
            return new Mx2Mt210Creator();
        } else if("pacs.002.001".equals(messageType)){
            return new Mx2MtN99Creator();
        } else if("camt.060.001".equals(messageType)){
            return new Mx2Mt920Creator();
        } else {
            throw new SwiftException("Invalid message type");
        }
    }

    /**
     * ①如果messageType = pacs.004.001,需要特殊处理
     * TxInf.OrgnlGrpInf.OrgnlMsgNmId 如果是以 pacs.008或者MT103开始，就是MT103RETURN
     * 如果是以 pacs.009或者MT202开始，就是MT202RETURN
     * 如果是以 pacs.009或者MT205开始，就是MT205RETURN
     * 如果 TxInf.OrgnlGrpInf.OrgnlMsgNmId 没有值，
     *     TxInf/RtrChain/Dbtr/Agt 有值 or TxInf/RtrChain/Cdtr/Agt 有值 就是MT202RETURN
     *     否则 就是MT103RETURN
     * @param document
     * @param messageType
     * @return
     */
    private String handlingMessageType(Document document, String messageType) {
        if ("pacs.004.001".equals(messageType)) {
            Map<String, String> parentElementMaps = XmlUtil.getParentElementMaps(document);
            String grpParentElementName = parentElementMaps.get("OrgnlGrpInf");
            String orgMessageNameId = XmlUtil.getXmlNodeValue(document, grpParentElementName + ".OrgnlGrpInf.OrgnlMsgNmId");
            if (StringUtil.isNotEmpty(orgMessageNameId)) {
                if (orgMessageNameId.indexOf("pacs.008") > -1 || orgMessageNameId.startsWith("MT103")) {
                    messageType += ".MT103";
                } else if (orgMessageNameId.indexOf("pacs.009") > -1 || orgMessageNameId.startsWith("MT202")) {
                    messageType += ".MT202";
                } else if (orgMessageNameId.indexOf("pacs.009") > -1 || orgMessageNameId.startsWith("MT205")) {
                    messageType += ".MT205";
                }
            } else {
                String bodyParentElementName = parentElementMaps.get("TxInf");
                int dbtrAgtCount = XmlUtil.getChildrenCount(document, bodyParentElementName + ".TxInf.RtrChain.Dbtr.Agt", null);
                int cdtrAgtCount = XmlUtil.getChildrenCount(document, bodyParentElementName + ".TxInf.RtrChain.Cdtr.Agt", null);
                if (dbtrAgtCount > 0 && cdtrAgtCount > 0) {
                    messageType += ".MT202";
                } else {
                    messageType += ".MT103";
                }
            }
        }
        return messageType;
    }

    /**
     * 如果传入的xml 没有报文头，导致dom4j无法解析
     * 提取xml里面的AppHdr和Document，重新组装成有效的xml报文
     */
    private String handlingXml(String xmlStr) {
        boolean flag = true;
        try {
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            documentBuilder.parse(xmlStr);
        } catch (Exception e) {
            flag = false;
        }
        if (flag) { //如果是有效xml报文，直接返回
            return xmlStr;
        } else {
            return SwiftTransfer.fetchXmlFromMixMessage(xmlStr);
        }
    }
}
